// Export list
const name1 = 1
const name2 = 1
const name5 = 1
const variable1 = 1
const variable2 = 1
const variable3 = 1
const name6 = 1
export { name1, name2 }
export { variable1 as name3, variable2 as name4, /* …, */ name5 }
export { variable3 as 'name' }
export { name6 as default /*, … */ }
