// This replicates the way the in-the-wild `got` module does things:
// https://github.com/sindresorhus/got/blob/3822412/source/index.ts

class got {
  foo = 'foo'
}

export default got
export { got }
export * from './something.mjs'
export { default as renamedDefaultExport } from './lib/baz.mjs'
