// These export identifiers are only supported in Node 16+
exports['one.two'] = () => console.log('b')

// See: https://github.com/nodejs/import-in-the-middle/issues/94
exports['unsigned short'] = 'something'

exports._ = 'foo'
exports.$ = 'bar'
